<?php

namespace App\Services\Settlement;

use App\Domicilio;
use App\DomicilioHasSettlement;
use App\LogSender;
use App\Services\Domicilio\DomicilioServices;
use App\Settlement;
use App\Status;
use Barryvdh\DomPDF\Facade as PDF;
use Illuminate\Support\Facades\Auth;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;
use App\Traits\ApiResponser;
use App\Enums\DomicilioStatusConst;
use App\Http\Resources\Settlements\SettlementResource;

class SettlementService
{
    use ApiResponser;

    protected $domiciliosService;

    public function __construct(DomicilioServices $domicilioServices)
    {
        $this->domiciliosService = $domicilioServices;
    }

    /* public function listDomiciliosSettlements(
        int $perPage,
        string $from,
        string $to,
        string $search,
        ?int $senderId = null
    ) {
        try {
            $columns = [
                'settlements.id',
                'settlements.created_at',
                'settlements.deleted_at',
                'users.name as created_by_name',
            ];

            $query = Settlement::query()
                ->with('createdBy', 'domicilios.logSender') // Incluye la relación con logSender.
                ->when($search, function ($query) use ($search) {
                    $query->where(function ($q) use ($search) {
                        $q->where('id', 'like', "%{$search}%")
                            ->orWhere('pdf_url', 'like', "%{$search}%");
                    });
                })
                ->when($from && $to, function ($query) use ($from, $to) {
                    $query->whereBetween('created_at', [$from, $to]);
                })
                ->when($senderId, function ($query) use ($senderId) {
                    $query->whereHas('domicilios', function ($q) use ($senderId) {
                        $q->where('log_sender_id', $senderId);
                    });
                })
                ->orderBy('created_at', 'DESC');

            return SettlementResource::collection($query->paginate($perPage));
        } catch (\Exception $e) {
            return $this->errorResponse($e->getMessage(), 400);
        }
    } */

    public function listDomiciliosSettlements(
        int $perPage,
        string $from,
        string $to,
        string $search,
        ?int $senderId = null
    ) {

        try {
            $columns = [
                'settlements.id',
                'settlements.created_at',
                'settlements.deleted_at',
                'users.name as created_by_name',
            ];

            $query = Settlement::query()
                ->with('createdBy', 'domicilios.logSender', 'domicilios.dropshipper') // Incluye la relación con dropshipper.
                ->when($search, function ($query) use ($search) {
                    $query->where(function ($q) use ($search) {
                        $q->where('id', 'like', "%{$search}%")
                            ->orWhere('pdf_url', 'like', "%{$search}%");
                    });
                })
                ->when($from && $to, function ($query) use ($from, $to) {
                    $query->whereBetween('created_at', [$from, $to]);
                })
                ->when($senderId, function ($query) use ($senderId) {
                    $query->whereHas('domicilios', function ($q) use ($senderId) {
                        $q->where('log_sender_id', $senderId);
                    });
                })
                ->orderBy('created_at', 'DESC');

            return SettlementResource::collection($query->paginate($perPage));
        } catch (\Exception $e) {
            return $this->errorResponse($e->getMessage(), 400);
        }
    }

    public function getSettlementInformation(int $id)
    {
        $settlement = Settlement::findOrFail($id);
        $dataSettlement = $this->generatePdfData($settlement, $settlement->domicilios);

        return $dataSettlement;
    }

    /**
     * @throws \Exception
     */
    public function getPDFByDomicilio(int $id, $domicilios, $senderId = null)
    {
        $settlement = Settlement::findOrFail($id);
        $sender = null;

        if ($senderId) {
            $sender = LogSender::findOrFail($senderId);
        }

        $pdfUrl = $this->emitPDF($settlement, $domicilios, $sender);
        $settlement->pdf_url = $pdfUrl;
        $settlement->save();

        return $settlement->pdf_url;
    }

    /*
     * METODO PARA MARCAR DOMICILIOS COMO LIQUIDOS
     * Y REGISTRARLOS EN SETTLEMENTS PARA AGRUPARLOS
     */
    public function changeDomiciliosToSettlement($items)
    {
        if (!isset($items) || count($items) === 0) {
            throw new BadRequestException("No hay un array correcto de domicilios");
        }

        $settlement = Settlement::create([
            "created_by" => Auth::id(),
            "modified_by" => Auth::id()
        ]);

        foreach ($items as $item) {
            $this->domiciliosService->changeStatusPaid($item["domicilio_id"], true);
            DomicilioHasSettlement::create([
                "domicilio_id" => $item["domicilio_id"],
                "settlement_id" => $settlement->id
            ]);
        }

        $settlement->save();

        return $settlement;
    }

    public function emitPDF($settlement, $domicilios, $sender = null)
    {
        $configuration = \App\Configuration::all()->first();

        $pdfData = $this->generatePdfData($settlement, $domicilios, $sender);

        $domicilios = collect($pdfData->domicilios);


        // Calcular los totales
        $totalCollectionValue = $domicilios->sum('collection_value');
        $totalCollectionCommission = $domicilios->sum('collection_comission');
        $totalFullfilment = $domicilios->sum('fullfilment');
        $totalLastMille = $domicilios->sum('last_mille');
        $totalTotal = $domicilios->sum('total');
        $totalOperationCost = isset($pdfData->has_dropshipper) ? $this->sumOperationCost($pdfData) : 0;

        // Pasar los totales a la vista
        $pdf = PDF::loadView('onx.settlement', compact(
            'pdfData',
            'configuration',
            'totalCollectionValue',
            'totalCollectionCommission',
            'totalFullfilment',
            'totalLastMille',
            'totalTotal',
            'totalOperationCost'
        ))->setPaper('a4', 'landscape');

        $path = public_path('pdf/');
        $fileName =  'settlement_' . $settlement->id . '.pdf';

        // Crear la carpeta si no existe
        if (!file_exists($path)) {
            mkdir($path, 0777, true); // 0777 es el permiso para que el directorio sea legible/escribible/ejecutable
        }

        $pdf->save($path . '/' . $fileName);
        $file = public_path() . "/pdf/" . $fileName;
        if (ENV('APP_ENV') == 'production') {
            $url = url('pdf');
        } else {
            $url = url('pdf');
        }
        if (strpos(url('/'), 'ecomdex') !== false) {
            $url = url('api/public/pdf');
        }
        return "{$url}/{$fileName}";
    }

    public function removeSettlementDomicilio(int $id)
    {
        $settlement = Settlement::findOrFail($id);
        $domicilios = $settlement->domicilios;

        foreach ($domicilios as $domicilio) {
            $domicilio->liquidated = 0;
            $domicilio->save();
            DomicilioHasSettlement::where("domicilio_id", $domicilio->id)
                ->delete();
        }

        $settlement->delete();

        return $settlement;
    }

    public function removeDomicilioInSettlement(int $id)
    {
        $domicilio = Domicilio::findOrFail($id);
        $relation = DomicilioHasSettlement::where("domicilio_id", $domicilio->id)->first();
        $settlement = $relation->settlement;
        $relation->delete();

        if ($settlement->domicilios->count() == 0) {
            $settlement->delete();
        }

        $domicilio->liquidated = 0;
        $domicilio->save();

        return $domicilio;
    }

    private function getDeliveredDate($domicilioId)
    {
        $status = Status::where("domicilio_id", $domicilioId)
            ->where("status", DomicilioStatusConst::DELIVERED)
            ->get()
            ->last();
        if ($status) {
            return $status->created_at->format('Y/m/d');
        }

        return null;
    }

    private function calculateLasMille($finalPrice, $discount)
    {
        $discount = $discount / 100;
        return $finalPrice - ($finalPrice * $discount);
    }

    private function generateDataDropshipper($pdfData, $dropshipperSettlement, $finallyPrice)
    {
        $pdfData->provider_cost = $dropshipperSettlement->provider_cost;
        $pdfData->name_dropshipper = $dropshipperSettlement->name_dropshipper;
        $pdfData->total_dropshipper = $pdfData->collection_value - $pdfData->provider_cost;
        $pdfData->com_collection_provider = $pdfData->provider_cost * 0.03;
        $pdfData->com_collection_dropshipper = $pdfData->total_dropshipper * 0.03;
        $pdfData->dropshipper_settlement = $pdfData->total_dropshipper - $pdfData->com_collection_dropshipper - $finallyPrice;
        $pdfData->provider_settlement = $pdfData->provider_cost - $pdfData->com_collection_provider - $pdfData->fullfilment;
        $pdfData->logistic_cost = $pdfData->fullfilment + $pdfData->com_collection_dropshipper + $pdfData->com_collection_provider + $finallyPrice;

        return $pdfData;
    }

    private function isThereDropshipperInformation($pdfData)
    {

        foreach ($pdfData->domicilios as $domicilio) {
            if (isset($domicilio->name_dropshipper)) {
                return true;
            }
        }

        return false;
    }

    private function generatePdfData($settlement, $domicilios, $sender = null)
    {
        $pdfData = new \stdClass();
        $domiciliosArray = array();
        $pdfData->id = $settlement->id;

        if ($sender) {
            $pdfData->sender = $sender->name;
        }

        foreach ($domicilios as $domicilio) {

            $domicilio = Domicilio::findOrFail($domicilio["id"]);
            $obj = new \stdClass();
            $obj->reference = $domicilio->reference;
            $obj->status = $domicilio->status;
            $obj->date = $domicilio->created_at->format('Y/m/d');
            $obj->delivery_date = $this->getDeliveredDate($domicilio->id);
            $obj->city = $domicilio->zone->city->name;
            $obj->sender = $domicilio->logSender->name;
            $obj->collection_value = $domicilio->total;
            $obj->collection_comission = $domicilio->total * 0.03;
            $obj->fullfilment = $domicilio->logSender->fullfilment ?? 1.00;

            $obj->last_mille = $this->calculateLasMille(
                $domicilio->delivery,
                $domicilio->logSender->discount
            );

            $obj->total = ($obj->collection_value - $obj->collection_comission) - $obj->fullfilment - $obj->last_mille;

            if ($domicilio->dropshipper->count() > 0) {
                $dropshipperSettlement = $domicilio->dropshipper->first();
                $obj = $this->generateDataDropshipper($obj, $dropshipperSettlement, $domicilio->delivery, $obj->fullfilment);
            }

            array_push($domiciliosArray, $obj);
        }

        $pdfData->domicilios = $domiciliosArray;
        if ($this->isThereDropshipperInformation($pdfData)) {
            /*
                SE PONE UN FLAG PARA DETERMINAR MAS RAPIDO
                Y MAS FACIL SI EL REPORTE DEBE TENER INFORMACION DROPSHIPPER
            */

            $pdfData->has_dropshipper = true;
        }
        $pdfData->created_at = $settlement->created_at;
        $pdfData->totalDomicilios = count($domiciliosArray);

        return $pdfData;
    }

    private function sumOperationCost($pdfData)
    {
        $total = 0;

        foreach ($pdfData->domicilios as $domicilio) {
            $total += $domicilio->logistic_cost;
        }

        return $total;
    }
}
